### Global variables
MKI_VERSION = 0.0.1
GLOBAL_PREFIX ?= /usr/share/mkimage

CONFIGDIR = $(GLOBAL_PREFIX)
TOOLSDIR = $(GLOBAL_PREFIX)/tools

TOPDIR ?= $(CURDIR)

################################################################################
### Global helpers
################################################################################
RUN_MAKE		= $(TOOLSDIR)/mki-run-make
COUNT_CPU		= $(TOOLSDIR)/mki-count-cpu
CHROOT_MKAPTBOX		?= $(TOOLSDIR)/mki-mkaptbox
CHROOT_CACHE		?= $(TOOLSDIR)/mki-cache
CHROOT_INVALIDATE_CACHE	?= $(TOOLSDIR)/mki-invalidate-cache

CHROOT_BUILD_PROPAGATOR	?= $(TOOLSDIR)/mki-build-propagator
CHROOT_SCRIPTS		?= $(TOOLSDIR)/mki-scripts
CHROOT_IMAGE_SCRIPTS	?= $(TOOLSDIR)/mki-image-scripts
CHROOT_PREPARE 		?= $(TOOLSDIR)/mki-prepare
CHROOT_IMAGE_PREPARE	?= $(TOOLSDIR)/mki-image-prepare
CHROOT_PACK 		?= $(TOOLSDIR)/mki-pack
CHROOT_CLEAN 		?= $(TOOLSDIR)/mki-clean
CHROOT_EXEC 		?= $(TOOLSDIR)/mki-exec
CHROOT_RUN 		?= $(TOOLSDIR)/mki-run
CHROOT_COPY_SUBDIRS	?= $(TOOLSDIR)/mki-copy-subdirs
CHROOT_COPY_PKGS	?= $(TOOLSDIR)/mki-copy-pkgs
CHROOT_COPY_TREE	?= $(TOOLSDIR)/mki-copy-tree
CHROOT_COPY_ISOLINUX	?= $(TOOLSDIR)/mki-copy-isolinux
CHROOT_COPY_PXELINUX	?= $(TOOLSDIR)/mki-copy-pxelinux
CHROOT_INSTALL		?= $(TOOLSDIR)/mki-install
CHROOT_IMAGE_INSTALL	?= $(TOOLSDIR)/mki-image-install

################################################################################
### Config global variables
################################################################################
GLOBAL_TARGET ?=
GLOBAL_HSH_NUMBER ?=
GLOBAL_HSH_APT_CONFIG ?=
GLOBAL_HSH_APT_PREFIX ?=
GLOBAL_HSH_USE_QEMU ?=
GLOBAL_HSH_LANGS ?=

GLOBAL_QUIET ?=
GLOBAL_VERBOSE ?=

ifdef GLOBAL_QUIET
    GLOBAL_VERBOSE =
endif

HSH_CACHEDIR = $(CACHEDIR)/hsh

################################################################################
### Per-image variables
################################################################################
WORKDIRNAME	= .work
OUTDIRNAME	= .out
CACHEDIRNAME	= .cache

MYMAKEFILE 	= $(CURDIR)/Makefile
WORKDIR 	= $(CURDIR)/$(WORKDIRNAME)
CACHEDIR	= $(WORKDIR)/$(CACHEDIRNAME)
PKGBOX		= $(WORKDIR)/pkgbox

OUTDIR		?= $(WORKDIR)/$(OUTDIRNAME)

PROFILE ?=
SUBDIRS ?=
CLEANUP_OUTDIR ?= 1

TARGET = $(GLOBAL_TARGET)

SYSTEM_ARCH = $(shell uname -m)
ifeq ($(shell uname -m),x86_64)
    TARGET ?= $(SYSTEM_ARCH)
else
    TARGET ?= i586
endif

QUIET = $(GLOBAL_QUIET)
VERBOSE = $(GLOBAL_VERBOSE)

HSH_NUMBER = $(GLOBAL_HSH_NUMBER)
HSH_APT_CONFIG = $(GLOBAL_HSH_APT_CONFIG)
HSH_APT_PREFIX = $(GLOBAL_HSH_APT_PREFIX)
HSH_USE_QEMU = $(GLOBAL_HSH_USE_QEMU)
HSH_LANGS = $(GLOBAL_HSH_LANGS)

NO_CACHE =

MKI_SCRIPTDIR = $(CURDIR)/scripts.d
MKI_IMAGE_SCRIPTDIR = $(CURDIR)/image-scripts.d

# Instrumental chroot packages list or filename with packages list.
CHROOT_PACKAGES =

# Work chroot packages list.
IMAGE_PACKAGES =

MKI_DESTDIR =

# Specifies a directory to copy in to $(SUBWORKDIR) as is.
COPY_TREE =

### Pack image
# Defines a way of results packing ($(SUBWORKDIR) as root).
#
# boot    - create a bootable ISO image.
# squash  - create a squashfs filesystem.
# tarbz2  - create a tar compessed by bzip2.
# isoboot - create a bootable ISO image (low level).
# isodata - create a data ISO image.
# data    - copy $(SUBWORKDIR) as is.
#
MKI_PACKTYPE =

# Output image name (useful for squash, tarbz2, isoboot and isodata types).
MKI_OUTNAME ?=

# Use <number> processors.
PACK_SQUASHFS_PROCESSORS ?= $(shell $(COUNT_CPU))

###
### A variables for 'boot' pack type and 'build-propagator' target.
###

# Modules list for propagator.
PROPAGATOR_MAR_MODULES =

# Specifies a propagator initfs file. It is required for creation of file system.
PROPAGATOR_INITFS =

# Specifies a propagator top message.
PROPAGATOR_VERSION =

###
### 'boot' type variables
###

# Available 'isolinux' and 'pxelinux' types.
BOOT_TYPE =

###
### 'boot'/'isoboot' type variables
###

# Specifies a text string that will be written into the volume header. See: man mkisofs(8).
BOOT_APPI =

# Specifies the Copyright file name. See: man mkisofs(8).
BOOT_COPY =

# Specifies the abstract file name. See: man mkisofs(8).
BOOT_ABST =

# Specifies the bibliographic file name. See: man mkisofs(8).
BOOT_BIBL =

# Specifies a text string that will be written into the volume header. See: man mkisofs(8).
BOOT_PREP =

# Specifies  a  text  string that will be written into the volume header. See: man mkisofs(8).
BOOT_PUBL =

# Specifies  the  system  ID. See: man mkisofs(8).
BOOT_SYSI =

# Specifies  the  volume  ID  (volume name or label) to be written into the master block. See: man mkisofs(8).
BOOT_VOLI =

# Specifies the volset ID. See: man mkisofs(8).
BOOT_VOLS =

################################################################################
### Rules
################################################################################

.PHONY: $(SUBDIRS)
.EXPORT_ALL_VARIABLES:
.NOTPARALLEL:

all: profile $(SUBDIRS)

prepare: $(SUBDIRS)
	mkdir -p -- $(PKGBOX) $(WORKDIR) $(OUTDIR)
	mkdir -p -- $(CACHEDIR)/mki $(CACHEDIR)/hsh

prepare-workdir: prepare $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    $(CHROOT_PREPARE) && \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

prepare-image-workdir: prepare-workdir $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    $(CHROOT_IMAGE_PREPARE) && \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

profile:
	if [ -n "$(PROFILE)" ]; then \
	    $(CHROOT_INVALIDATE_CACHE) mki; \
	    [ ! -d "$(WORKDIR)" ] || $(CHROOT_CLEAN); \
	fi

run-scripts: prepare-workdir $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    if ! $(CHROOT_SCRIPTS); then \
		$(CHROOT_INVALIDATE_CACHE) mki; \
		exit 1; \
	    fi; \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

run-image-scripts: prepare-workdir $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    if ! $(CHROOT_IMAGE_SCRIPTS); then \
		$(CHROOT_INVALIDATE_CACHE) mki; \
		exit 1; \
	    fi; \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

build-propagator: prepare-workdir $(SUBDIRS)
	$(CHROOT_BUILD_PROPAGATOR)

copy-isolinux: prepare-workdir $(SUBDIRS)
	$(CHROOT_COPY_ISOLINUX)

copy-pxelinux: prepare-workdir $(SUBDIRS)
	$(CHROOT_COPY_PXELINUX)

copy-tree: prepare-workdir $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    $(CHROOT_COPY_TREE) && \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

copy-subdirs: prepare-workdir $(SUBDIRS)
	if [ -n "$(SUBDIRS)" ] && ! $(CHROOT_CACHE) check $@; then \
	    $(CHROOT_COPY_SUBDIRS) && \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

copy-packages: prepare-workdir $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    $(CHROOT_COPY_PKGS) && \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

build-image: prepare-image-workdir $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    $(CHROOT_IMAGE_INSTALL) $(IMAGE_PACKAGES) && \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

pack-image: prepare-workdir copy-subdirs $(SUBDIRS)
	if ! $(CHROOT_CACHE) check $@; then \
	    $(CHROOT_PACK) && \
	    $(CHROOT_CACHE) build $@ || exit 1; \
	fi

clean-current:
	$(CHROOT_INVALIDATE_CACHE) mki; \
	[ ! -d "$(WORKDIR)" ] || $(CHROOT_CLEAN);

clean: clean-current $(SUBDIRS)

distclean-current: clean-current
	rm -rf -- $(WORKDIR) $(OUTDIR) $(CACHEDIR) $(PKGBOX)

distclean: distclean-current $(SUBDIRS)

$(SUBDIRS):
	$(RUN_MAKE) $(MAKE) $(MFLAGS) -C $@ $(MAKECMDGOALS)
